/* Cdigo para el clculo de la energa de Fermi y concentracin
   de portadores en semiconductores.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define SEMICONDUCTOR  "SILICIO"
#define DONADORES      "Antimonio"
#define ACEPTORES      "Boro"

#define E_G  1.2        /* Energa del intervalo prohibido en eV      */
#define KT   0.0258     /* Constante Boltzmann * T en eV para T=300 K */
#define N_C 28.80       /* DOS efectiva en BC 10^{18}cm^{-3}          */
#define N_V 10.80       /* DOS efectiva en BV 10^{18}cm^{-3}          */
#define N_D  0.001      /* Concentracin donadores 10^{18}cm^{-3}     */
#define N_A  0.010      /* Concentracin aceptores 10^{18}cm^{-3}     */
#define E_D  0.039      /* Energa ionizacin de donadores en eV      */
#define E_A  0.045      /* Energa ionizacin de aceptores en eV      */
#define G_C  2.         /* Degeneracin de espn para donadores       */
#define G_V  4.         /* Degeneracin de espn para aceptores       */

#define ETA_MIN -50.0   /* Valor mnimo inicial de eta                */
#define ETA_MAX -20.0   /* Valor mximo inicial de eta                */
#define EPS      1e-6   /* Precisin en el valor de eta               */
#define N_MAX    30     /* Nmero mximo de iteraciones               */
#define FILENAME  "DATOS.DAT"

double fermi(double eta) { /* Integral de Fermi                       */
   double r;
   if(eta<=-5.) return exp(eta);
   if(eta>-5.0 && eta<=0.7) {
      r=exp(eta);
      r+=-0.3*r*r+0.06*r*r*r;
      return r;
   }
   if(eta>0.7 && eta<=1.1) {
      r=exp(eta);
      r/=(1.0+0.27*r);
      return r;
   }
   if(eta>1.1 && eta<=1.3) {
      r=0.75*sqrt(eta*eta*eta)*(1.0+1.15/eta/eta);
      return r;
   }
   if(eta>1.3) {
      r=0.75*sqrt(eta*eta*eta)*(1.0+1.23/eta/eta);
      return r;
   }
}

double funcion(double eta, double eg) {
   double r, auxd, auxa;
   auxd=(E_G-E_D)/KT+eta;
   auxa=(E_A-E_G)/KT-eta;
   auxd=N_D/(1.+G_C*exp(auxd));
   auxa=N_A/(1.+G_V*exp(auxa));
   r=N_V*fermi(-eg-eta)-N_C*fermi(eta)+auxd-auxa;
   return r;
}

void main(void) {
   double etamin=ETA_MIN,etamax=ETA_MAX,etamitad,eps=1e30;
   double ef,eg=E_G/KT,n,p;
   int i=0;
   FILE *out;
   out=fopen(FILENAME,"w");
   if(funcion(etamin,eg)*funcion(etamax,eg)>0.) {
      fprintf(out,"La solucin est fuera del intervalo elegido\n");
      exit(1);
   }
   while(eps>EPS) {             /* Mtodo de la biseccin             */
      i++;
      if(i==N_MAX) {            /* Posible mala convergencia          */
         printf("Demasiadas iteraciones en el proceso\n");
         exit(1);
      }
      etamitad=0.5*(etamax+etamin);
      if(funcion(etamin,eg)*funcion(etamitad,eg)<0.) etamax=etamitad;
      else etamin=etamitad;
      eps=etamax-etamin;        /* Longitud del nuevo intervalo       */
   }
   etamitad=0.5*(etamax+etamin);
   ef=KT*etamitad+E_G;
   n=N_C*fermi(etamitad);
   p=N_V*fermi(-eg-etamitad);
   fprintf(out,"Semiconductor... %s\n\n",SEMICONDUCTOR);
   fprintf(out,"(todas las concentraciones en cm^{-3})\n\n");
   fprintf(out,"Donadores....... %s\n",DONADORES);
   fprintf(out,"Concentracin donadores  %6.2e\n\n",N_D*1e18);
   fprintf(out,"Aceptores....... %s\n", ACEPTORES);
   fprintf(out,"Concentracin aceptores  %6.2e\n\n",N_A*1e18);
   fprintf(out,"Concentracin electrones %6.2e\n\n",n*1e18);
   fprintf(out,"Concentracin huecos     %6.2e\n\n",p*1e18);
   fprintf(out,"Energa de Fermi %8.6lf eV\n",ef);
   fclose(out);
}
